@@ -1,4 +1,3 @@
-# coding: utf-8
 # Copyright (c) Scanlon Materials Theory Group
 # Distributed under the terms of the MIT License.
 
@@ -22,9 +21,10 @@ from sumo.electronic_structure.bandstructure import (
     get_projections_by_branches,
 )
 from sumo.plotting import (
+    colorline,
+    draw_themed_line,
     pretty_plot,
     pretty_subplot,
-    rgbline,
     styled_plot,
     sumo_base_style,
     sumo_bs_style,
@@ -64,10 +64,53 @@ class SBSPlotter(BSPlotter):
             self.bs = self._bs
             self.nbands = self._nb_bands
 
+    @staticmethod
+    def _reset_zero_energy(bs_plot_data, zero_energy=0.0):
+        """Modify reference energy of data from bs_plot_data
+
+        This method is defined in Pymatgen and obtains plotting data with zero
+        set to 0. (if zero_to_efermi=False), self.efermi (if
+        zero_to_efermi=True and metallic) or the absolute VBM energy
+        (zero_to_efermi=False and insulating).
+
+        Sumo allows other reference energies to be specified (e.g. from
+        sc-fermi); conveniently the data dictionary from Pymatgen contains a
+        "zero_energy" field so the data can always be related to absolute (DFT)
+        values.
+
+        Args:
+            bs_plot_data (:obj:`dict`): data dictionary from
+                pymatgen.electronicstructure.plotter.BSPlotter.bs_plot_data()
+            zero_energy (:obj:`float`, Optional): New reference energy
+
+        Returns:
+            dict in format of bs_plot_data.
+        """
+
+        shifted_data = {}
+        energy_shift = bs_plot_data["zero_energy"] - zero_energy
+
+        for key, value in bs_plot_data.items():
+            if key in ("vbm", "cbm"):
+                shifted_data[key] = [(pt[0], pt[1] + energy_shift) for pt in value]
+            elif key == "energy":
+                shifted_data["energy"] = {}
+                for spin, energies in value.items():
+                    shifted_data["energy"][spin] = [
+                        array + energy_shift for array in energies
+                    ]
+            elif key == "zero_energy":
+                shifted_data[key] = zero_energy
+            else:
+                shifted_data[key] = value
+        return shifted_data
+
     @styled_plot(sumo_base_style, sumo_bs_style)
     def get_plot(
         self,
         zero_to_efermi=True,
+        zero_line=False,
+        zero_energy=None,
         ymin=-6.0,
         ymax=6.0,
         width=None,
@@ -95,8 +138,14 @@ class SBSPlotter(BSPlotter):
         valence bands and orange lines indicates conduction bands.
 
         Args:
-            zero_to_efermi (:obj:`bool`): Normalise the plot such that the
-                valence band maximum is set as 0 eV.
+            zero_to_efermi (:obj:`bool`): Shift the plot such that the
+                Fermi energy of the band structure data is plotted at 0 eV.
+                Note that this "efermi" is for the benefit of Pymatgen and may
+                not reflect the actual Fermi level; in Sumo it has usually
+                already been shifted to the VBM.
+            zero_line (:obj:`bool`, optional): Draw a horizontal line at zero
+            zero_energy (:obj:`float`, optional): Zero energy reference. (If
+                unset, defaults to VBM or Fermi Energy as appropriate.)
             ymin (:obj:`float`, optional): The minimum energy on the y-axis.
             ymax (:obj:`float`, optional): The maximum energy on the y-axis.
             width (:obj:`float`, optional): The width of the plot.
@@ -194,7 +243,10 @@ class SBSPlotter(BSPlotter):
             plt = pretty_plot(width=width, height=height, dpi=dpi, plt=plt)
             ax = plt.gca()
 
-        data = self.bs_plot_data(zero_to_efermi)
+        data = self.bs_plot_data(zero_to_efermi=True)
+        if zero_energy is not None:
+            data = self._reset_zero_energy(data, zero_energy=zero_energy)
+
         dists = data["distances"]
         eners = data["energy"]
 
@@ -245,11 +297,13 @@ class SBSPlotter(BSPlotter):
                 ax.plot(dists[nd], e, c="C0", linestyle="--", zorder=2)
 
         self._maketicks(ax, ylabel=ylabel)
+
         self._makeplot(
             ax,
             plt.gcf(),
             data,
             zero_to_efermi=zero_to_efermi,
+            zero_line=zero_line,
             vbm_cbm_marker=vbm_cbm_marker,
             width=width,
             height=height,
@@ -260,6 +314,7 @@ class SBSPlotter(BSPlotter):
             plot_dos_legend=plot_dos_legend,
             dos_label=dos_label,
             aspect=aspect,
+            spin=spin,
         )
         return plt
 
@@ -270,9 +325,15 @@ class SBSPlotter(BSPlotter):
         mode="rgb",
         normalise="all",
         interpolate_factor=4,
+        color1="#FF0000",
+        color2="#00FF00",
+        color3="#0000FF",
+        colorspace="lab",
         circle_size=150,
         projection_cutoff=0.001,
+        zero_energy=None,
         zero_to_efermi=True,
+        zero_line=False,
         ymin=-6.0,
         ymax=6.0,
         width=None,
@@ -355,11 +416,23 @@ class SBSPlotter(BSPlotter):
                 lines). A larger number indicates greater interpolation.
             circle_size (:obj:`float`, optional): The area of the circles used
                 when ``mode = 'stacked'``.
+            color1 (str): A color specified in any way supported by matplotlib. Used
+                when ``mode = 'rgb'``.
+            color2 (str): A color specified in any way supported by matplotlib. Used
+                when ``mode = 'rgb'``.
+            color3 (str): A color specified in any way supported by matplotlib. Used
+                when ``mode = 'rgb'``.
+            colorspace (str): The colorspace in which to perform the interpolation. The
+                allowed values are rgb, hsv, lab, luvlc, lablch, and xyz. Used
+                when ``mode = 'rgb'``.
             projection_cutoff (:obj:`float`): Don't plot projections with
                 intensities below this number. This option is useful for
                 stacked plots, where small projections clutter the plot.
             zero_to_efermi (:obj:`bool`): Normalise the plot such that the
                 valence band maximum is set as 0 eV.
+            zero_line (:obj:`bool`, optional): Draw a horizontal line at zero
+            zero_energy (:obj:`float`, optional): Zero energy reference. (If
+                unset, defaults to VBM or Fermi Energy as appropriate.)
             ymin (:obj:`float`, optional): The minimum energy on the y-axis.
             ymax (:obj:`float`, optional): The maximum energy on the y-axis.
             width (:obj:`float`, optional): The width of the plot.
@@ -432,8 +505,8 @@ class SBSPlotter(BSPlotter):
                 style. This can make alternative styles behave more
                 predictably.
             spin (:obj:`Spin`, optional): Plot a spin-polarised band structure,
-                "up" or "1" for spin up only, "down" or "-1" for spin down only.
-                Defaults to ``None``.
+                "up" or "1" for spin up only, "down" or "-1" for spin down
+                only. Defaults to ``None``.
 
         Returns:
             :obj:`matplotlib.pyplot`: The projected electronic band structure
@@ -460,7 +533,10 @@ class SBSPlotter(BSPlotter):
             plt = pretty_plot(width, height, dpi=dpi, plt=plt)
             ax = plt.gca()
 
-        data = self.bs_plot_data(zero_to_efermi)
+        data = self.bs_plot_data(zero_to_efermi=zero_to_efermi)
+        if zero_energy is not None:
+            data = self._reset_zero_energy(data, zero_energy=zero_energy)
+
         nbranches = len(data["distances"])
 
         # Ensure we do spin up first, then spin down
@@ -470,6 +546,7 @@ class SBSPlotter(BSPlotter):
                 "Spin-selection only possible with spin-polarised "
                 "calculation results"
             )
+
         if spin is Spin.up:
             spins = [spins[0]]
         elif spin is Spin.down:
@@ -522,7 +599,7 @@ class SBSPlotter(BSPlotter):
             if mode == "rgb":
 
                 # colours aren't used now but needed later for legend
-                colours = ["#ff0000", "#00ff00", "#0000ff"]
+                colours = [color1, color2, color3]
 
                 # if only two orbitals then just use red and blue
                 if len(weights) == 2:
@@ -530,13 +607,14 @@ class SBSPlotter(BSPlotter):
                     colours = ["#ff0000", "#0000ff"]
 
                 ls = "-" if spin == Spin.up else "--"
-                lc = rgbline(
+                lc = colorline(
                     distances,
                     bands,
-                    weights[0],
-                    weights[1],
-                    weights[2],
-                    alpha=1,
+                    weights.transpose((1, 2, 0)),
+                    color1=color1,
+                    color2=color2,
+                    color3=color3,
+                    colorspace=colorspace,
                     linestyles=ls,
                     linewidth=(rcParams["lines.linewidth"] * 1.25),
                 )
@@ -561,7 +639,7 @@ class SBSPlotter(BSPlotter):
                         distances,
                         bands,
                         c=c,
-                        s=circle_size * w ** 2,
+                        s=circle_size * w**2,
                         zorder=z,
                         rasterized=True,
                     )
@@ -571,7 +649,7 @@ class SBSPlotter(BSPlotter):
             if isinstance(spec, str):
                 label = spec
             else:
-                label = "{} ({})".format(spec[0], " + ".join(spec[1]))
+                label = f"{spec[0]} ({' + '.join(spec[1])})"
             ax.scatter([-10000], [-10000], c=c, s=50, label=label, edgecolors="none")
 
         if dos_plotter:
@@ -596,6 +674,7 @@ class SBSPlotter(BSPlotter):
             plt.gcf(),
             data,
             zero_to_efermi=zero_to_efermi,
+            zero_line=zero_line,
             vbm_cbm_marker=vbm_cbm_marker,
             width=width,
             height=height,
@@ -606,6 +685,7 @@ class SBSPlotter(BSPlotter):
             dos_label=dos_label,
             plot_dos_legend=plot_dos_legend,
             aspect=aspect,
+            spin=spin,
         )
         return plt
 
@@ -615,6 +695,7 @@ class SBSPlotter(BSPlotter):
         fig,
         data,
         zero_to_efermi=True,
+        zero_line=False,
         vbm_cbm_marker=False,
         ymin=-6.0,
         ymax=6.0,
@@ -625,13 +706,16 @@ class SBSPlotter(BSPlotter):
         dos_label=None,
         plot_dos_legend=True,
         aspect=None,
+        spin=None,
     ):
         """Tidy the band structure & add the density of states if required."""
-        # draw line at Fermi level if not zeroing to e-Fermi
-        if not zero_to_efermi:
-            ytick_color = rcParams["ytick.color"]
+        if zero_line:
+            draw_themed_line(0, ax)
+
+        # Draw line at Fermi level if not zeroing to e-Fermi
+        elif not zero_to_efermi:
             ef = self.bs.efermi
-            ax.axhline(ef, color=ytick_color)
+            draw_themed_line(ef, ax)
 
         # set x and y limits
         ax.set_xlim(0, data["distances"][-1][-1])
@@ -652,13 +736,22 @@ class SBSPlotter(BSPlotter):
             if not dos_options:
                 dos_options = {}
 
-            dos_options.update({"xmin": ymin, "xmax": ymax})
+            dos_options.update(
+                {
+                    "xmin": ymin,
+                    "xmax": ymax,
+                    "zero_energy": data["zero_energy"],
+                    "zero_to_efermi": False,
+                }
+            )
             self._makedos(
                 ax,
                 dos_plotter,
                 dos_options,
                 dos_label=dos_label,
                 plot_legend=plot_dos_legend,
+                zero_line=zero_line,
+                spin=spin,
             )
         else:
             # keep correct aspect ratio for axes based on canvas size
@@ -675,7 +768,16 @@ class SBSPlotter(BSPlotter):
 
                 ax.set_aspect(aspect * ((x1 - x0) / (y1 - y0)))
 
-    def _makedos(self, ax, dos_plotter, dos_options, dos_label=None, plot_legend=True):
+    @staticmethod
+    def _makedos(
+        ax,
+        dos_plotter,
+        dos_options,
+        dos_label=None,
+        plot_legend=True,
+        zero_line=False,
+        spin=None,
+    ):
         """This is basically the same as the SDOSPlotter get_plot function."""
 
         # don't use first 4 colours; these are the band structure line colours
@@ -686,14 +788,19 @@ class SBSPlotter(BSPlotter):
         mask = plot_data["mask"]
         energies = plot_data["energies"][mask]
         lines = plot_data["lines"]
-        spins = [Spin.up] if len(lines[0][0]["dens"]) == 1 else [Spin.up, Spin.down]
+        if spin is None:
+            spins = [Spin.up] if len(lines[0][0]["dens"]) == 1 else [Spin.up, Spin.down]
+        elif isinstance(spin, Spin):
+            spins = [spin]
+        else:
+            spins = spin
 
         # disable y ticks for DOS panel
         ax.tick_params(axis="y", which="both", right=False)
 
         for line_set in plot_data["lines"]:
             for line, spin in it.product(line_set, spins):
-                if spin == Spin.up:
+                if spin == Spin.up or len(spins) == 1:
                     label = line["label"]
                     densities = line["dens"][spin][mask]
                 else:
@@ -709,12 +816,18 @@ class SBSPlotter(BSPlotter):
                 )
                 ax.plot(densities, energies, label=label, color=line["colour"])
 
-            # x and y axis reversed versus normal dos plotting
-            ax.set_ylim(dos_options["xmin"], dos_options["xmax"])
+        # x and y axis reversed versus normal dos plotting
+        ax.set_ylim(dos_options["xmin"], dos_options["xmax"])
+        if len(spins) == 1:
+            ax.set_xlim(0, plot_data["ymax"])
+        else:
             ax.set_xlim(plot_data["ymin"], plot_data["ymax"])
 
-            if dos_label is not None:
-                ax.set_xlabel(dos_label)
+        if zero_line:
+            draw_themed_line(0, ax)
+
+        if dos_label is not None:
+            ax.set_xlabel(dos_label)
 
         ax.set_xticklabels([])
         if plot_legend:
@@ -733,7 +846,7 @@ class SBSPlotter(BSPlotter):
 
     @classmethod
     def _sanitise_label_group(cls, labelgroup):
-        """Implement label hacks: Hide trailing @, remove label with leading @
+        r"""Implement label hacks: Hide trailing @, remove label with leading @
 
         Labels split with $\mid$ symbol will be treated for each part.
         """
@@ -741,7 +854,7 @@ class SBSPlotter(BSPlotter):
         if r"$\mid$" in labelgroup:
             label_components = labelgroup.split(r"$\mid$")
             good_labels = [
-                l for l in map(cls._sanitise_label, label_components) if l is not None
+                i for i in map(cls._sanitise_label, label_components) if i is not None
             ]
             if len(good_labels) == 0:
                 return None
@@ -770,7 +883,7 @@ class SBSPlotter(BSPlotter):
                     # If a branch connection, check all parts of label
                     if r"$\mid$" in temp_ticks[i][1]:
                         label_components = temp_ticks[i][1].split(r"$\mid$")
-                        good_labels = [l for l in label_components if l[0] != "@"]
+                        good_labels = [i for i in label_components if i[0] != "@"]
                         if len(good_labels) == 0:
                             continue
                         else:
@@ -789,7 +902,7 @@ class SBSPlotter(BSPlotter):
 
         logging.info("Label positions:")
         for dist, label in list(zip(unique_d, unique_l)):
-            logging.info("\t{:.4f}: {}".format(dist, label))
+            logging.info(f"\t{dist:.4f}: {label}")
 
         ax.set_xticks(unique_d)
         ax.set_xticklabels(unique_l)
